use Test::More qw( no_plan );

use Sub::Scheduler;
use Time::HiRes qw( sleep time );

my $flag = 0;
sub flag { $flag++ }

eval { add_task(); };
like($@, qr/add_task: usage/, 'add_task throw exception');

eval { add_task(\&flag); };
like($@, qr/add_task: usage/, 'add_task throw exception');

eval { add_task("not a sub ref", time()); };
like($@, qr/add_task: usage/, 'add_task throw exception');

eval { add_task(\&flag, time(), 0); };
like($@, qr/add_task: usage/, 'add_task throw exception');

my $taskid = add_task(\&flag, time()+0.02);
ok(defined $taskid, 'add_task returned taskid');

eval { del_task($taskid); };
is($@, '', q{del_task don't throw exception});
$taskid = add_task(\&flag, time()+0.02);

sleep 0.01;
run_tasks();
is($flag, 0, 'flag() was not executed');

sleep 0.01;
run_tasks();
is($flag, 1, 'flag() was executed');

my $next = get_next_run_time();
is($next, undef, 'no more tasks to run');

eval { del_task($taskid); };
like($@, qr/del_task: no such task/, 'del_task throw exception');

my ($flag1, $flag2) = (0, 0);
sub flag1 { $flag1++; }
sub flag2 { $flag2++; }
my $time = time();
my $task1 = add_task(\&flag1, $time+0.05);
my $task2 = add_task(\&flag2, $time+0.01, 0.02);

$next = get_next_run_time();
is($next, $time+0.01, 'flag2 will be next task in 0.01 sec');

sleep 0.01;
run_tasks();
is($flag2, 1, 'flag2 was executed first time');
sleep 0.01;
run_tasks();
is($flag2, 1, 'flag2 was not executed after 0.02 sec');
sleep 0.01;
run_tasks();
is($flag2, 2, 'flag2 was executed second time');
sleep 0.01;
run_tasks();
is($flag2, 2, 'flag2 was not executed after 0.04 sec');
is($flag1, 0, 'flag1 was not executed');
sleep 0.01;
$time = time();
run_tasks($time);
is($flag1, 1, 'flag1 was executed first time');
is($flag2, 3, 'flag2 was executed third time');

$next = get_next_run_time();
is($next, $time+0.02, 'flag2 will be next task in 0.02 sec');

